<?php

namespace App\Factory;

class ProductFactory {
    
    private $connection;
    private $util;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {
        $this->connection = $connection;
        $this->util       = $util;
    }

    public function getProduct ($product) {
        $sql = "
            SELECT CDPRODUTO, CDARVPROD, NMPRODUTO, IDTIPOCOMPPROD
              FROM PRODUTO PR
             WHERE CDPRODUTO  = :CDPRODUTO
                OR CDPRODINTE = :CDPRODINTE
                OR CDARVPROD  = :CDARVPROD
        ";

        $productCode = str_replace(".", "", $product);
        $productCode = preg_replace("/[^0-9A-Z]/", "", $productCode);
        

        $params = [
            ':CDPRODUTO'  => $productCode,
            ':CDPRODINTE' => $productCode,
            ":CDARVPROD"  => $productCode
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function insertProduct ($product, $generalParams, $orderData) {
        $sql = "
            INSERT INTO ITCOMANDAVEN
    			(CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN,
    			CDPRODUTO, QTPRODCOMVEN, VRPRECCOMVEN, TXPRODCOMVEN,
    			IDSTPRCOMVEN, VRDESCCOMVEN, NRLUGARMESA, DTHRINCOMVEN,
    			IDPRODIMPFIS, CDLOJA, VRACRCOMVEN, NRSEQPRODCOM, NRSEQPRODCUP,
    			VRPRECCLCOMVEN, CDCAIXACOLETOR, CDPRODPROMOCAO, CDVENDEDOR, NRINSCRCONS, DSOBSPEDDIGCMD, CDSENHAPED)
    		VALUES
    			(:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN,
    			:CDPRODUTO, :QTPRODCOMVEN, :VRPRECCOMVEN, :TXPRODCOMVEN,
    			:IDSTPRCOMVEN, :VRDESCCOMVEN, :NRLUGARMESA, TO_DATE(:CURRDATE, 'dd/mm/yyyy hh24:mi:ss'),
    			:IDPRODIMPFIS, :CDLOJA, :VRACRCOMVEN, :NRSEQPRODCOM, :NRSEQPRODCUP,
    			:VRPRECCLCOMVEN, :CDCAIXACOLETOR, :CDPRODPROMOCAO, :CDVENDEDOR, :NRINSCRCONS, :DSOBSPEDDIGCMD, :CDSENHAPED)";

    	if(isset($generalParams["SGESTADO"]))	
    	    date_default_timezone_set($this->util->getCurrentTimezone($generalParams["SGESTADO"]));
    	
    	$params = [
    	    ":CDFILIAL"       => $product["CDFILIAL"],
			":NRVENDAREST"    => $product["NRVENDAREST"],
			":NRCOMANDA"      => $product["NRCOMANDA"],
			":NRPRODCOMVEN"   => $product["NRPRODCOMVEN"],
			":CDPRODUTO"      => $product["CDPRODUTO"],
			":QTPRODCOMVEN"   => $this->util->toFloat($product["QTPRODCOMVEN"]),
			":VRPRECCOMVEN"   => $this->util->toFloat($product["VRPRECCOMVEN"]),
			":TXPRODCOMVEN"   => $product["TXPRODCOMVEN"],
			":IDSTPRCOMVEN"   => $product["IDSTPRCOMVEN"],
			":VRDESCCOMVEN"   => $this->util->toFloat($product["VRDESCCOMVEN"]),
			":NRLUGARMESA"    => $product["NRLUGARMESA"],
			":CURRDATE"       => date("d/m/Y H:i:s"),
			":IDPRODIMPFIS"   => $product["IDPRODIMPFIS"],
			":CDLOJA"         => $product["CDLOJA"],
			":VRACRCOMVEN"    => $this->util->toFloat($product["VRACRCOMVEN"]),
			":NRSEQPRODCOM"   => $product["NRSEQPRODCOM"],
			":NRSEQPRODCUP"   => $product["NRSEQPRODCUP"],
			":VRPRECCLCOMVEN" => $this->util->toFloat($product["VRPRECCLCOMVEN"]),
			":CDCAIXACOLETOR" => $product["CDCAIXACOLETOR"],
			":CDPRODPROMOCAO" => $product["CDPRODPROMOCAO"],
			":CDVENDEDOR"     => $product["CDVENDEDOR"],
			":NRINSCRCONS"    => $product["NRINSCRCONS"],
			":DSOBSPEDDIGCMD" => $product["DSOBSPEDDIGCMD"],
			":CDSENHAPED"     => $product["CDSENHAPED"]
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function getObservation ($observation, $generalParams, $isAcr = false) {
    	if($isAcr){
    	    $sql = "
        		SELECT *
        		FROM OCORRENCIA
        		WHERE CDGRPOCOR = :CDGRPOCOR
        		  AND CDOCORR   = :CDOCORR
        		  AND IDCONTROLAOBS = 'A'
        		  AND CDPRODUTO IS NOT NULL";
    	} else{
    	    $sql = "
        		SELECT *
        		FROM OCORRENCIA
        		WHERE CDGRPOCOR = :CDGRPOCOR
        		  AND CDOCORR   = :CDOCORR
        		  AND IDCONTROLAOBS IN ('O', 'R')";
    	}
    	
    	$obs = explode("|", $observation);
    	
    	$params = [
    		":CDGRPOCOR" => $obs[0],
    		":CDOCORR"   => $obs[1]
    	];
    	
    	return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function insertObservation ($observation, $generalParams, $orderData) {

        $sql = "
            INSERT INTO	OBSITCOMANDAVEN
                (CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN, CDOCORR, CDGRPOCOR, NRPRODCOMVENOBS, QTPRODCOMVENOBS)
            VALUES
                (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN, :CDOCORR, :CDGRPOCOR, :NRPRODCOMVENOBS, :QTPRODCOMVENOBS)";
		
	    $params = [
	        ":CDFILIAL"        => $generalParams["CDFILIAL"],
            ":NRVENDAREST"     => $orderData["NRVENDAREST"],
            ":NRCOMANDA"       => $orderData["NRCOMANDA"],
            ":NRPRODCOMVEN"    => $observation["NRPRODCOMVEN"],
            ":NRPRODCOMVENOBS" => $observation["NRPRODCOMVENOBS"],
            ":CDOCORR"         => $observation["CDOCORR"],
            ":CDGRPOCOR"       => $observation["CDGRPOCOR"],
            ":QTPRODCOMVENOBS" => $observation["QTPRODCOMVENOBS"]  
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function getProductFamilyDetails ( $generalParams) {
        $sql = "SELECT CDFILIAL, CDFAMILISALD, CDPRODUTO, QTCONSDIAFAM, IDPERMREFEXTRA 
                   FROM FAMSALDOPROD 
                    WHERE CDFILIAL =  :CDFILIAL
                     AND CDPRODUTO = :CDPRODUTO";
                    
         $params = [
            ":CDPRODUTO"  => $generalParams["CDPRODUTO"],
            ":CDFILIAL"   => $generalParams["CDFILIAL"]
         ];
         
        $famProditem = $this->connection->fetchAll($sql,  $params);
        
        
        if (empty($famProditem))
            return null;
       
        return $famProditem;
        
    }
    
    public function getCombinedConfTela($CDFILIAL, $CDCAIXA) {
        $sqlCombinedConfTela = "
            SELECT NRCONFTELA 
    		  FROM CONFTELACAIXA
    		 WHERE CDFILICAIXA = :CDFILIAL
    		   AND CDCAIXA  = :CDCAIXA
    		   AND DTINIVIGENCIA = (SELECT DTINIVIGENCIA FROM (SELECT DTINIVIGENCIA 
    		    					  FROM CONFTELACAIXA 
                                     WHERE SYSDATE >= DTINIVIGENCIA 
    					               AND CDFILICAIXA = :CDFILIAL 
    					               AND CDCAIXA  = :CDCAIXA
                                  ORDER BY DTINIVIGENCIA DESC) WHERE ROWNUM = 1)";
                              
        $params = [
            ':CDFILIAL' => $CDFILIAL,
            ':CDCAIXA'  => $CDCAIXA
        ];
        
        $result = $this->connection->fetchAll($sqlCombinedConfTela, $params);
        return $result[0]['NRCONFTELA'];
    }
    
    public function getCombinedProducts($CDFILIAL, $CDLOJA, $NRCONFTELA) {
        $sqlCombinedProds = "
            SELECT I.NRBUTTON, I.DSBUTTON, I.NRLIMPRODCOM, L.IDTIPCOBRA
              FROM ITMENUCONFTE I
              JOIN LOJA L ON L.CDFILIAL = I.CDFILIAL
             WHERE I.CDFILIAL = :CDFILIAL
               AND L.CDLOJA = :CDLOJA
               AND I.NRCONFTELA = :NRCONFTELA
               AND I.IDTPBUTTON = 'B'
          ORDER BY I.NRBUTTON
        ";
        
        $params = [
            ':CDFILIAL'   => $CDFILIAL,
            ':CDLOJA'     => $CDLOJA,
            ':NRCONFTELA' => $NRCONFTELA
        ];
        
        // Gets the combined product "buttons", which are the containers for the actual combined products.
        return $this->connection->fetchAll($sqlCombinedProds, $params);
    }
    
    public function getCombinedChilds($CDFILIAL, $NRCONFTELA) {
        $sqlCombinedChilds = "
            SELECT NVL(I.DSBUTTON, P.NMPRODUTO) AS NMPRODUTO, P.CDARVPROD, P.CDPRODUTO, P.IDTIPOPROD, P.IDTIPOCOMPPROD
              FROM PRODUTO P LEFT JOIN ITMENUCONFTE I
                                    ON P.CDPRODUTO = I.CDIDENTBUTON
                                   AND I.CDFILIAL = :CDFILIAL
                                   AND I.NRCONFTELA = :NRCONFTELA
            WHERE P.IDTIPOPROD IS NOT NULL
        ";
        
        $params = [
            ':CDFILIAL' => $CDFILIAL,
            ':NRCONFTELA'  => $NRCONFTELA
        ];
        
        // Gets all the actual combined products. These must be sorted into their respective buttons.
        return $this->connection->fetchAll($sqlCombinedChilds, $params);
    }
}
